#include "KS0108.h"
#include <util\delay.h>
#include "PIC1.h"
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

//PIC1_BMP[]

unsigned char image_part[32];
unsigned char homes[32];		//location of every part sorted by ids
const uint8_t *pic;
unsigned char empty_id = 31;

#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3
#define RESET 4
#define SAVE 5
#define LOAD 6
#define NONE 7

#define KEY_PIN PINA
#define UP_MASK (1<<0)
#define DOWN_MASK (1<<2)
#define LEFT_MASK (1<<1)
#define RIGHT_MASK (1<<3)
#define RESET_MASK (1<<4)
#define SAVE_MASK (1<<5)
#define LOAD_MASK (1<<6)

char move(unsigned char side);
/**********************************************/
//this methode cut 16*16 pixel from image and
//copy it to image_part
void cut_part(const uint8_t *pic , unsigned char id)
{
	unsigned char i , j;
	uint8_t* ptr = image_part;

	if(id == 31)
	{
		for(i = 0; i < 32; i++ , ptr++)
			*ptr = 0;
		return;
	}

	pic += ((id%8)*16) + ((id/8)*256);
	for(i = 0; i != 2 ; i++)
	{
		for(j = 0; j != 16; j++)
		{
			*ptr = pgm_read_byte(pic);
			ptr++;
			pic++;
		}

		pic += 128-16;
	}
}
/***********************************************/
//this methode draw image_part to id
void draw_part(unsigned char id)
{
	unsigned char x , y;

	x = ((id%8) * 16);
	y = (id/8) * 2;
	glcd_drawImage(image_part , x , y , 16 ,2);
}

void draw_all()
{
	unsigned char i;
	for(i = 0; i < 32 ; i++)
	{
		cut_part( pic , homes[i] );
		draw_part( i );
	}
}
/************************************************/
//this method beham parts in partsLoc[]
static uint8_t rnd_table[256] PROGMEM = {
4,173,100,101,85,149,200,67,41,61,8,232,179,221,127,90,223,
172,0,216,233,183,243,122,237,226,6,240,140,174,175,211,157,
19,204,180,44,203,96,181,35,135,208,36,159,39,178,89,112,
215,106,195,75,185,119,114,142,26,20,22,201,177,164,34,197,
48,222,51,130,16,120,199,202,109,13,68,10,137,92,1,189,
246,160,93,150,147,47,154,60,156,217,253,134,205,88,64,184,
129,59,126,110,27,124,54,42,32,66,207,143,115,70,235,91,
99,190,252,72,169,117,46,94,3,187,210,128,83,162,52,82,
77,214,248,37,192,111,186,18,23,196,241,229,148,176,107,7,
79,105,167,250,30,62,144,65,71,97,17,11,24,102,103,238,
230,139,55,163,155,209,254,56,25,95,236,191,213,123,78,152,
228,73,193,14,225,125,69,104,239,220,198,212,165,251,245,166,
161,21,206,247,249,158,98,219,40,170,58,57,151,244,168,188,
86,45,218,133,63,136,81,224,146,118,53,227,87,76,74,131,
132,28,141,33,49,50,194,138,182,121,231,15,116,108,43,234,
145,113,29,38,242,2,5,31,171,80,12,84,153,9 };

unsigned char rnd_index = 0;

char rnd(unsigned char min , unsigned char max)
{
	rnd_index++;
	if(min == max)
		return 0;
	return (pgm_read_byte(rnd_table+rnd_index)%(max-min)) + min;
}

void beham()
{
	unsigned char i , t ,last=10;
	for( i = 0; i < 32; i++)
		homes[i] = i;
	draw_all();
	for(i = 0; i < 200; i++)
		_delay_ms(10);
	for( i = 0 ; i != 0xff; )
	{
		t = rnd(0,4);
		if( (t == UP && last != DOWN) ||
			(t == DOWN && last != UP) ||
			(t == LEFT && last != RIGHT) ||
			(t == RIGHT && last != LEFT) )
		{
			move( t );
			i++;
			last = t;
		}
	}
}
/************************************************/
//this method move home
char move(unsigned char side)
{
	unsigned char x ,y;
	x = empty_id%8;
	y = empty_id/8;
	switch(side)
	{
		case DOWN:
			if(y > 0)
				y--;
			else
				return 0;
			break;
		case UP:
			if(y < 3)
				y++;
			else
				return 0;
			break;
		case RIGHT:
			if(x > 0)
				x--;
			else
				return 0;
			break;
		case LEFT:
			if(x < 7)
				x++;
			else
				return 0;
	}
	
	homes[ empty_id ] = homes[ x + (y*8) ];
	homes[ x + (y*8) ] = 31;
	empty_id = x + (y*8);
	draw_all();
	return 1;
}

unsigned char last_btn = 5;
char bottons()
{
	if( (KEY_PIN&UP_MASK) == 0 ) //up
	{
		if(last_btn != UP )
		{
			last_btn = UP;
			return UP;
		}
		return NONE;
	}
	if( (KEY_PIN&DOWN_MASK) == 0 ) //down
	{
		if(last_btn != DOWN )
		{
			last_btn = DOWN;
			return DOWN;
		}
		return NONE;
	}
	if( (KEY_PIN&LEFT_MASK) == 0 ) //left
	{
		if(last_btn != LEFT )
		{
			last_btn = LEFT;
			return LEFT;
		}
		return NONE;
	}
	if( (KEY_PIN&RIGHT_MASK) == 0 ) //right
	{
		if(last_btn != RIGHT )
		{
			last_btn = RIGHT;
			return RIGHT;
		}
		return NONE;
	}
	if( (KEY_PIN&RESET_MASK) == 0 ) //right
	{
		return RESET;
	}

	if( (KEY_PIN&SAVE_MASK) == 0 ) //save
	{
		return SAVE;
	}

	if( (KEY_PIN&LOAD_MASK) == 0 ) //load
	{
		return LOAD;
	}

	last_btn = NONE;
	return NONE;

}

void Save()
{
	eeprom_write_block((void*)image_part , (void*)0 ,32 );
	eeprom_write_block((void*)homes , (void*)32 ,32 );
	eeprom_write_byte((void*)64 , empty_id);
}

void Load()
{
	eeprom_read_block((void*)image_part , (void*)0 ,32 );
	eeprom_read_block((void*)homes , (void*)32 ,32 );
	empty_id = eeprom_read_byte((void*)64);
	draw_all();
}

int main()
{
	char b;

	_delay_ms(100);
	PORTA = 0xff;

	while(1)
	{
		empty_id = 31;
		b = NONE;
		//
		glcd_init();

		glcd_drawImage_P(shamsipour_bmp,0,0,128,8);
		while(bottons() == NONE)
			rnd(0,0);

		pic = PIC1_BMP;

		beham();

		while(b != RESET)
		{
			b = bottons();
			if(b <= 3)
				move( b );
			if(b == LOAD)
			{
				Load();
				while(bottons() != NONE);
			}
			if(b == SAVE)
			{
				Save();
				while(bottons() != NONE);
			}
			
			rnd(0,0);
		}
	}
}
